#ifndef __EXTRACTION_H
#define __EXTRACTION_H



#include "util.H"
#include "matx.H"
#include <algorithm>
#include <vector>


/**
 * @class Permet d'extraire les coordonnées des touchés d'une image
 */
class Extraction{

private:

	// image correspondant à un doigt pour le blockMatching
	Matrx pattern;

	AFFICHAGE_FACE affichage; //Permet d'afficher une frame
	int temps_affichage; //Permet d'afficher une frame

	double niveauSeuil; // le niveau du seuil 
	double precision; // La précision à atteindre
	double rayonMin; // Le rayon des doigts
	double rayonMax;

	
	double rayonMoyenneGradient;
	double nbrPointsEchantillon;
	double seuilGradient; // Le gradient minimum des contours du doigt

	int depthGradient; // le type d'image du gradient

	int nbIterMax; // Le nombre maximum d'itérations autorisées

	bool faceMilieu; // epsilon à ajouter par rapport au max de l'image



	// Matrices utilisées par différents algorithmes pour éviter les allocations répétées
	Matrx lissee;
	Matrx propre;
	Matrx gradient;
	Matrx seuil;
	Matrx distTransform;
	Matrx whatever;



	// Meme chose...
	Matrx grad_x;
	Matrx grad_y;
	Matrx abs_grad_x, abs_grad_y;
	Matrx projection;



	std::vector<cv::Point2d> candidats;

protected:

	/**
	 * @brief lissage par convolution avec une gaussienne de taille taille
	 */
	void floutter(Matrx & img, Matrx & dest, cv::Size taille);

	/**
	 * @brief Permet d'extraire grossièrement les taches pouvant correspondre à des doigts
	 */
	void trouverCandidats(Matrx & imageRedressee);

	/**
	 * @brief Calcul la norme du gradient de l'image en norme 1
	 */
	void calculerGradient(Matrx & img, Matrx & dest);

	/**
	 * @brief Effectue une transformée en distance
	 */
	void distTrans(Matrx & img, Matrx & dest);


	/**
	 * @brief Calcul le barycentre d'une tache à partir de ça position aproximative et de l'image seuillée
	 */
	cv::Point2d extraireBary(cv::Point2d & pts, Matrx & imageRedressee);

	/**
	 * @brief Meme chose avec une pondération selon l'intensité des pixels
	 */
	void extraireBaryPondIntens(std::vector<cv::Point2d> & pts, Matrx & imageRedressee);

	/**
	 * @brief effectue un seuillage (constant ou non selon la face) de l'image
	 */
	void seuiller(Matrx & img, Matrx & dest);



public:

	/**
	 * @brief Barycentre avec une pondération gaussienne
	 */
	cv::Point2d extraireBaryPondGauss(cv::Point2d & pts, Matrx & imageRedressee, Matrx & gauss);

	/**
	 * Algorithme de block matching subpixel, à partir d'une position de départ d'un pas min et max et d'une matrice "à matcher"
	 */
	cv::Point2d blockMatching(Matrx & img, Matrx & sub_image, cv::Point2d depart, double precision, double precisionDepart);
	 

	/**
	 * @brief Configure et alloue la mémoire necessaire à l'extraction des points
	 */
	void configurer(Matrx & imageRedressee, bool faceM);

	/**
	 * @brief Configure l'affichage d'une certaine étape de l'algorithme
	 */
	void configurerAffichage(AFFICHAGE_FACE aff, unsigned int temps_aff);
	             
	/**
	 * @brief extrait de l'image redressée les points pts
	 */
	void extrairePoints(std::vector<cv::Point2d>& pts, Matrx & imageRedressee, CHOIX_ALGO algo, int r = 5);

	/**
	 * @brief Permet d'enlever les candidats non pertinents
	 */
	void trierCandidats(Matrx & img);

	/**
	 * @brief Permet de trouver l'ellipse qui fitte le mieux
	 * la projection de l'histogramme selon les x et les y 
	 */
	cv::Point2d extraireCentreEllipse(cv::Point2d & pts);


	/**
	 * Retourne une Roi aux bonnes dimensions selon la face
	 */
	cv::Rect getRectROI(cv::Point2d pt,Matrx & reg);


	/**
	 * @brief calcule la valeur médianne des pixels seuillés
	 */
	cv::Point2d median(cv::Point2d & pts);

};

#endif
