#ifndef __MATX_H
#define __MATX_H

#include <cv.h>
#include "util.H"
#include <fstream>
class Matrx;

#include "sourceVideo.H"

/**
 * @class Matrx 
 * @brief Etend le type cv::Mat
 */
class Matrx: public cv::Mat{
	public:

		clock_t temps;

		Matrx();

		// (_type is CV_8UC1, CV_64FC3, CV_32SC(12) etc.)
		Matrx(int _rows, int _cols, int _type = CV_8UC1);

		Matrx(SourceVideo2& srcvid);

		Matrx(const Matrx& m);

		// constructor for matrix headers pointing to user-allocated data
		Matrx(int _rows, int _cols, int _type, void* _data, size_t _step=AUTO_STEP);

		// creates a matrix header for a part of the bigger matrix
		Matrx(const Matrx& m, const cv::Range& rowRange, const cv::Range& colRange);

		Matrx(const Matrx& m, const cv::Rect& roi);

		Matrx(const cv::Mat & m);

		/**
		 * Je ne comprend pas pourquoi il n'y a pas de conversion implicite...
		 * Dans l'idéal il ne faudrait pas surcharger cette méthode.
		 * Si quelqu'un à une idée, sinon il faut bien la tester...
		 */
		Matrx operator=(cv::Mat const& mat);

		/**
		 * @brief Permet d'afficher l'image..
		 * @param temps_affichage le temps d'attente avant de continuer (1 par défaut
		 * 0 attendra un evenement clavier avant de continuer
		 */
		void afficher(int temps_affichage1); // Affiche la matrice m

		/**
		 * @brief accesseur rapide
		 */
		template<typename T>
		inline T& getPx(int i, int j){
			return ((T*)data)[cols*i+j];
		}

		/**
		 * @brief Calcul la moyenne des pixels voisins de centre à une distance donnée
		 */
		template<typename T>
		double moyennePxDisque(cv::Point2d centre, double distance, int nbr_echantillons = 10);
		void initGauss(cv::Size taille);

	// initialiase la matrice pour le barycentre pondéré gaussien

	template<typename T>
	inline double getSubPx(double i, double j){

		assert(isContinuous());
		double eps = 1e-10;
		
		int x1 = floor(i);
		int x2 = ceil(i+eps); //
		int y1 = floor(j);
		int y2 = ceil(j+eps);
		

		return 
			(x2-i) * (y2-j) * ((T*)data)[cols*x1+y1] +
			(i-x1) * (y2-j) * ((T*)data)[cols*x2+y1] +
			(x2-i) * (j-y1) * ((T*)data)[cols*x1+y2] +
			(i-x1) * (j-y1) * ((T*)data)[cols*x2+y2];
	}
	
	// histogramme projetté selon X
	Matrx getProjectedHistoX();
	
	// histogramme projeté selon Y
	Matrx getProjectedHistoY();

	void histo();

	void printInFile();



	// sb = seuil bas de l'image
	// sh = seuil haut de l'image
	// renvoie la matrice seuille
	//Matrx seuilLineaire(int sb, int sh);

};

void seuilLineaire(Matrx & src, Matrx & dest, int sb, int sh);

template<typename T>
double moyenneEcartsCarres(Matrx& img, Matrx& sub_img, cv::Point2d position);

#include "matx.txx"



#endif

