#ifndef __SOURCEVIDEO_H
#define __SOURCEVIDEO_H

#include <cv.h>
#include <cstring>
#include <highgui.h>

class SourceVideo2;

#include "matx.H"
#include "sourceVideo.H"


extern "C" {
#include "grabber.h"
}

/**
 * @brief Le type de source video : 
 *
 * 3 possibilitées : un enregistrement, une camera Allied Vision, une simple webcam
 */
enum TYPE_SOURCE
	{
		SRC_FILE, // utilise un fichier video
		SRC_CAM, // utilise la premiere camera disponible
		SRC_DC // les cameras firewire
	};
	



class SourceVideo2{

private:
	TYPE_SOURCE type; // le mode de video

	cv::VideoCapture cap; // le moteur video opencv

	int nbLignes;
	int nbColonnes;



public:


	/**
	 * @brief SourceVideo constructeur à partir d'une ligne de commande
	 * @param argc le nombre d'argument de la ligne de commande
	 * @param argv les arguments
	 *
	 * Si un fichier est passé en argument, lecture du fichier, sinon lecture de la camera firewir
	 * et en dernier lieux, on se rabbat sur la premiere camera présente
	 */	
	SourceVideo2(int argc, char* argv[]);



	int configurerShutter(double moyenneAAtteindre);


	/**
	 * @brief pour dire à la camera de commener l'envoi des données
	 */
	void demarrer();

	/**
	 * @brief pour dire à la camera de commener l'envoi des données
	 */
	void arreter();

	/**
	 * @brief Affiche les caractéristiques 
	 */
	void reportFeatures();

	/**
	 * @brief prend une frame
	 *
	 */
	int dequeueFrame(Matrx * frame);

	/**
	 * @brief rend une frame
	 *
	 * La frame rendue est celle contenue dans frame.data
	 */
	void enqueueFrame(Matrx * frame);

	/**
	 * @brief Donne le nombre de ligne de l'image
	 */
	int getHauteur();

	/**
	 * @brief Donne le nombre de colonnes de l'image
	 */
	int getLargeur();

	/**
	 * @brief Destructeur : stop l'utilisation de la caméra
	 */ 
	~SourceVideo2();


	

};

#endif
