#ifndef __TACT_H
#define __TACT_H

#include <cv.h>

/**
 * @class Tact
 * @brief Représente un "touché" dans la durée
 */
class Tact{
private:


	bool absolue; // Si false, les motions sont détectés si l'écart est plus grand que 3*ecart_type

	double ecart_type; // L'écart type du bruit spatial associé à un doigt immobile

	cv::Point2d position; //La position du touché
	

	cv::Point2d deplacement; // deplacement depuis la dernière position connue
	
	cv::Vec2d vitesse; //la vitesse du doigt (en nombre de pixels par clock())

	clock_t temps; // Le temps où a été relevé la position

	int id; //L'identifiant du touché

	static int compteur; // le compteur des identifiants

public:

	/**
	 * @brief Accesseur sur la vitesse
	 */
	int getId();
	
	/**
	 * @brief Constructeur d'un nouveau contacte, de vitesse nulle
	 */
	Tact(cv::Point2d p, clock_t t);
	Tact();

	/**
	 * @brief Met à jour le contact avec les nouvelles cohordonnées.
	 * Calcul également la vitesse associée
	 *
	 * @return true si le point a changé de position
	 */
	bool maj(cv::Point2d& p, clock_t t);

	/**
	 * @brief Accesseur sur la position acutelle
	 */
	cv::Point2d getPos();

	/**
	 * @brief Accesseur sur l'évaluation de la prochaine position
	 * @param t le temps auquel on cherche la position
	 */
	cv::Point2d getPosPredict(clock_t t);

	/**
	 * @brief Accesseur sur le déplacement depuis la dernière position connue
	 */
	cv::Point2d getDeplacement();

	/**
	 * @brief Accesseur sur la vitesse
	 */
	cv::Vec2d getVitesse();


	/**
	 * @brief Affiche les informations relatives à un doigt
	 */
	void afficher();


	/**
	 * @brief : PROVISOIRE : retourne vrai si le point p est "le bon point suivant"
	 */ 
	bool estProche(cv::Point2d p, double distance_max);

	/**
	 * @brief retourne true si t==temps
	 */
	bool aEteMisAJour(clock_t t) const;
	
};

#endif
