#ifndef __TRACKER_H
#define __TRACKER_H


#include "eventsCubtile.H"
#include "tact.H"
#include "event.H"
#include "util.H"
/**
 * @class Tracker
 * @brief Implémente le suivis de points dans le plan
 */
class Tracker{

private:	
	// Les points actuels
	std::vector<Tact> ptsExistants;

	// La distance maximale entre un meme point sur deux frames successices
	double distanceMax;

	// Les evenements liés à ce tracker
	EventsCubtile evenements;

public:

	/**
	 * @brief Constructeur
	 * @param dist la distance maximale d'un même point sur deux frames successives
	 */
	Tracker(double dist = 15);

	/**
	 * Ne fait rien pour l'instant... Mais permet(ra) d'ajuster les paramètres : distance max, ...
	 */
	void configurer();


	/**
	 * Ne fait rien pour l'instant
	 */
	void configurerAffichage(AFFICHAGE_FACE aff, unsigned int temps_aff);

	/**
	 * @brief Met à jour le tracker avec les points de la nouvelles frame
	 * @param newPoints : les nouveaux vecteurs
	 * @param temps : le ticks auquel a été pris la frame
	 */
	void mettreAJour(std::vector<cv::Point2d>& newPoints, clock_t temps, ID_FACE idf, double scale);

	/**
	 * @brief Ajout d'évenements
	 */
	void addEventMotion(Event E);
	void addEventAppear(Event E);
	void addEventDisappear(Event E);

	/**
	 * @brief Lance et vide tous les évenements du buffer
	 */
	void signaler();

	/**
	 * Lance tous les évenements d'un buffer particulier
	 */
	void signalerMotion();
	void signalerAppear();
	void signalerDisappear();

};


#endif
