#ifndef __UTIL_H
#define __UTIL_H

#include "tact.H"


#define PI 3.14159265

/**
 * @brief pour la gestion des affichages
 */
enum AFFICHAGE_FACE{
	NONE, // Aucun affichage
	SEUIL, // L'image seuillée
	GRADIENT, // L'image représentative du gradient en norme 1
	DISTANCE, // Les transformées en distance
	CANDIDATS, //les positions des doigts detectés sans precisions
	TRACK, // L'image avec la position des doigts
	TRANSF, // l'image retournée (old API)
	ELIMINATION // les candidats éliminés
};


/**
 * @brief pour la gestion des algorithmes
 */
enum CHOIX_ALGO{
	BARYCENTRE,                 // Barycentre des composantes connexes seuillées
	BARYCENTRE_PONDERE_GAUSS,   // Barycentre utilisant une pondération gaussienne
	BARYCENTRE_VALEURS_PX,      // Barycentre utilisant la valeur des pixels normalisés
	ACANDIDATS,                 // En utilisant uniquement les positions grossières renvoyées par candidats
	XY_PROJ_ELLIPSE_FIT,	    // projection en X et Y des composantes connexes et estimation des coordonnées par une ellipse
	CUBTILE_DATA,               // modèle pré-enregistré d'un doigt, Peu efficace
	DISQUE,                     // fitting d'un doigt par un disque
	GRADIENT_FLOU,              //
	MEDIAN
};

/**
 * Identifier les différentes faces 
 */
enum ID_FACE{
	FACEH,
	FACEB,
	FACEM,
	FACED,
	FACEG
};



/**
 * Le type correspondant aux informations d'un évenements
 *
 * Une face, un identifiant de point et une position
 */
typedef struct _InfoEvent{
	ID_FACE face;
	int idPoint;
	cv::Point2d pos;

	inline _InfoEvent(ID_FACE idf, int idp, cv::Point2d p):
		face(idf),
		idPoint(idp),
		pos(p)
	{}

	inline void afficher(){
		std::cout<<"Face : "<<face<<" -  ID : "<<idPoint<<std::endl;
		std::cout<<pos<<std::endl;
	}

} InfoEvent;


/* Type utile dans le transformation 
 Redondance avec le type cv::Point2d mais attention, x et y inversés dans l'utilisation*/
typedef struct coordPix coordPix;

struct coordPix {
	double x;
	double y;
};

#endif
